package com.semarchy.mdm.client.samples;

import java.util.Locale;

import com.semarchy.mdm.client.IDataEditionConnection;
import com.semarchy.mdm.client.IMDMConnection;
import com.semarchy.mdm.client.MDMConnectionFactory;
import com.semarchy.mdm.runtime.activity.ActivityPriority;
import com.semarchy.mdm.runtime.activity.IActivityService;
import com.semarchy.mdm.runtime.activity.IActivityTask;
import com.semarchy.mdm.runtime.activity.IActivityService.ITaskCompletionResult;
import com.semarchy.mdm.runtime.commons.PageCriteria;
import com.semarchy.mdm.runtime.data.DataQuery;
import com.semarchy.mdm.runtime.data.IDataAccessService;
import com.semarchy.mdm.runtime.data.IEntityRow;
import com.semarchy.mdm.runtime.data.NewRowTarget;
import com.semarchy.mdm.runtime.model.dataaccess.IDataAccessModel;
import com.semarchy.mdm.runtime.model.dataaccess.IView;
import com.semarchy.mdm.runtime.model.logical.IApplication;
import com.semarchy.mdm.runtime.model.logical.IBusinessObject;
import com.semarchy.mdm.runtime.model.logical.IDETask;
import com.semarchy.mdm.runtime.model.logical.IDataEntryWorkflow;
import com.semarchy.mdm.runtime.model.logical.IEntity;
import com.semarchy.mdm.runtime.model.logical.IModel;

public class Sample5_ModifyEmployeeDataEntryWorkflow {

	public static void main(String[] args) {
		IMDMConnection conn = MDMConnectionFactory.connect(
				SamplesConstants.SEMARCHY_URL, SamplesConstants.USERNAME,
				SamplesConstants.PASSWORD);

		IDataEditionConnection dataEditionConnection = conn
				.createDataEditionConnection(SamplesConstants.GS_DATAEDITION);

		IModel logicalModel = dataEditionConnection.getLogicalModel();
		IApplication demoApplication = logicalModel.getApplication(SamplesConstants.GS_DEMO_APP);
		
		IDataAccessModel dataAccessModel = dataEditionConnection
				.getDataAccessModel();
		IEntity employeeEntity = logicalModel.getEntity("Employee");
		IDataEntryWorkflow modifyEmployeeWF = demoApplication.getDataEntryWorkflow("EmployeesCreationProcess");
		IActivityService activityService = dataEditionConnection
				.getActivityService();
		IDataAccessService dataService = dataEditionConnection
				.getDataAccessService();

		// Start a DataEntry Activity
		IActivityTask modifyEmployeeTask = activityService.startActivity(
				modifyEmployeeWF.getTask("CreateOrModifyEmployeeData"),
				"Client API Samples (ModifyEmployeeDataEntryWorkflowSample)",
				"Laura Bissot should now report to Neena Kochar.",
				ActivityPriority.HIGH, true, false, null);

		// Checkout record for Laura BISSOT into transaction from Master
		IView masterEmployeeView = dataAccessModel
				.getMasterView(employeeEntity);
		DataQuery checkoutQuery = new DataQuery(masterEmployeeView);
		checkoutQuery
				.setWhereCondition("FirstName = :FIRST_NAME and LastName = :LAST_NAME");
		checkoutQuery.setBinding("FIRST_NAME", "Laura");
		checkoutQuery.setBinding("LAST_NAME", "Bissot");
		checkoutQuery.setPageCriteria(new PageCriteria(0, 1));

		IDETask task = (IDETask) modifyEmployeeTask.getHumanTask();
		IBusinessObject employeeBO = task.getBOViews().get(0)
				.getBusinessObject();
		dataService.checkout(modifyEmployeeTask.getTaskUUID(), checkoutQuery,
				NewRowTarget.boRoot(employeeBO));

		// Query record in transaction
		IView dataEntryView = dataAccessModel.getDataEntryView(employeeEntity);
		DataQuery queryLauraBissotInTransaction = new DataQuery(dataEntryView, modifyEmployeeTask.getLoadID());
		queryLauraBissotInTransaction
				.setWhereCondition("FirstName = :FIRST_NAME and LastName = :LAST_NAME");
		queryLauraBissotInTransaction.setBinding("FIRST_NAME", "Laura");
		queryLauraBissotInTransaction.setBinding("LAST_NAME", "Bissot");
		IEntityRow lauraBissot = dataService.queryFirstEntityRow(queryLauraBissotInTransaction);

		DataQuery queryNeenaKocharID = new DataQuery(
				dataAccessModel.getDataEntryLookupView(employeeEntity), modifyEmployeeTask.getLoadID());
		queryNeenaKocharID
				.setWhereCondition("FirstName = :FIRST_NAME and LastName like :LAST_NAME");
		queryNeenaKocharID.setBinding("FIRST_NAME", "Neena");
		queryNeenaKocharID.setBinding("LAST_NAME", "Koc%");
		IEntityRow neenaKochar = dataService
				.queryEntityRows(queryNeenaKocharID).get(0);

		lauraBissot.setReference(dataEntryView.getManyToOneNode("Manager"),
				neenaKochar);

		dataService.persistRow(modifyEmployeeTask.getTaskUUID(), lauraBissot);

		// Complete Task and submit workflow 
		ITaskCompletionResult result = activityService.completeTask(modifyEmployeeTask.getTaskUUID(), 
				modifyEmployeeTask.getHumanTask().getSubmitTransition().getName(),								
				"OK. I'm done updating Laura Bissot.", null);
		
		if (result.isCompleted()) {
			System.out.println("Employee Modification submitted in batch number "
					+ result.getSubmittedBatchID());	
		} else {
			System.err.println("Task not completed transition failed: "
					+ result.getTransitionValidationReport().getSummary(Locale.getDefault()));
		}
	}
}
